# Copyright (c) 2024 Joseph Bishop

# EasyMeasure
# Script to measure objects in Maya. For more information visit: https://bishopvfx.gumroad.com/l/easymeasure

import maya.cmds as cmds
from PySide2.QtWidgets import QApplication, QDialog, QVBoxLayout, QCheckBox, QPushButton, QLabel, QMessageBox, QRadioButton

class AxisSelectionDialog(QDialog):
    def __init__(self, parent=None):
        super(AxisSelectionDialog, self).__init__(parent)
        self.setWindowTitle("EasyMeasure")
        self.selected_axes = ["x", "y", "z"]
        self.place_on_edges = True
        
        self.initUI()
        
    def initUI(self):
        layout = QVBoxLayout()

        description_label_1 = QLabel("1. Select the object to measure in the viewport.<br><br>2. Select the axes to measure:")
        layout.addWidget(description_label_1)
        
        self.checkbox_x = QCheckBox("X")
        self.checkbox_x.setChecked(True)
        self.checkbox_x.stateChanged.connect(self.updateSelectedAxes)
        layout.addWidget(self.checkbox_x)
        
        self.checkbox_y = QCheckBox("Y")
        self.checkbox_y.setChecked(True)
        self.checkbox_y.stateChanged.connect(self.updateSelectedAxes)
        layout.addWidget(self.checkbox_y)
        
        self.checkbox_z = QCheckBox("Z")
        self.checkbox_z.setChecked(True)
        self.checkbox_z.stateChanged.connect(self.updateSelectedAxes)
        layout.addWidget(self.checkbox_z)

        description_label_2 = QLabel("3. Place the measures on the edges or center<br>of the bounding box?")
        layout.addWidget(description_label_2)
        
        self.radio_edges = QRadioButton("Edges")
        self.radio_edges.setChecked(True)
        self.radio_edges.toggled.connect(self.updateMeasurementMode)
        layout.addWidget(self.radio_edges)
        
        self.radio_center = QRadioButton("Center")
        self.radio_center.setChecked(False)
        self.radio_center.toggled.connect(self.updateMeasurementMode)
        layout.addWidget(self.radio_center)
        
        self.btn_measure = QPushButton("Measure")
        self.btn_measure.clicked.connect(self.measureDimensions)
        layout.addWidget(self.btn_measure)
        
        self.setLayout(layout)
        
    def updateSelectedAxes(self):
        self.selected_axes = []
        if self.checkbox_x.isChecked():
            self.selected_axes.append("x")
        if self.checkbox_y.isChecked():
            self.selected_axes.append("y")
        if self.checkbox_z.isChecked():
            self.selected_axes.append("z")

    def updateMeasurementMode(self):
        self.place_on_edges = self.radio_edges.isChecked()
    
    def measureDimensions(self):
        selected_objects = cmds.ls(selection=True)
        if not selected_objects:
            # If no object is selected, show an error message
            QMessageBox.critical(self, "Error", "No object selected. Please select an object to measure.")
            return
    
        selected_object = selected_objects[0]
        dimensions = measure_dimensions(selected_object, self.selected_axes, self.place_on_edges)
        self.accept()

def measure_dimensions(selected_object, axes, place_on_edges):
    # Get bounding box of the selected object
    bbox = cmds.xform(selected_object, q=True, bb=True)
    
    # Get the object name
    object_name = cmds.ls(selected_object, shortNames=True)[0]
    
    # Create distance dimensions
    dimensions = []
    for axis in axes:
        axis_name = {"x": "Width", "y": "Height", "z": "Depth"}[axis]
        
        # Calculate start and end points
        start_point = [0, 0, 0]
        end_point = [0, 0, 0]
        if place_on_edges:
            # Place measurements on edges
            if axis == "x":
                start_point[0] = bbox[0]
                end_point[0] = bbox[3]
                start_point[1] = end_point[1] = bbox[4]
                start_point[2] = end_point[2] = bbox[5]
            elif axis == "y":
                start_point[1] = bbox[1]
                end_point[1] = bbox[4]
                start_point[0] = end_point[0] = bbox[3]
                start_point[2] = end_point[2] = bbox[5]
            elif axis == "z":
                start_point[2] = bbox[2]
                end_point[2] = bbox[5]
                start_point[0] = end_point[0] = bbox[3]
                start_point[1] = end_point[1] = bbox[4]
        else:
            # Place measurements in the center
            if axis == "x":
                start_point[0] = bbox[0]
                end_point[0] = bbox[3]
                start_point[1] = end_point[1] = (bbox[4] + bbox[1]) / 2
                start_point[2] = end_point[2] = (bbox[5] + bbox[2]) / 2
            elif axis == "y":
                start_point[1] = bbox[1]
                end_point[1] = bbox[4]
                start_point[0] = end_point[0] = (bbox[3] + bbox[0]) / 2
                start_point[2] = end_point[2] = (bbox[5] + bbox[2]) / 2
            elif axis == "z":
                start_point[2] = bbox[2]
                end_point[2] = bbox[5]
                start_point[0] = end_point[0] = (bbox[3] + bbox[0]) / 2
                start_point[1] = end_point[1] = (bbox[4] + bbox[1]) / 2
        
        # Create dimension node
        dimension_node = cmds.distanceDimension(sp=start_point, ep=end_point)
        
        # Define dimension transform node
        dimension_transform_node = cmds.listRelatives(dimension_node, parent=True)[0]
        
        # Rename the dimension transform node
        dimension_node_name = f"{object_name}_{axis_name}_measure"
        cmds.rename(dimension_transform_node, dimension_node_name)
        
        # Get the locator nodes created by the dimension node
        start_locator = cmds.listConnections(dimension_node_name + ".startPoint", s=True, d=False)[0]
        end_locator = cmds.listConnections(dimension_node_name + ".endPoint", s=True, d=False)[0]
        
        # Rename the locator nodes
        start_locator_name = f"{dimension_node_name}_start"
        end_locator_name = f"{dimension_node_name}_end"
        cmds.rename(start_locator, start_locator_name)
        cmds.rename(end_locator, end_locator_name)
        
        # Parent locators under the dimension node
        cmds.parent(start_locator_name, end_locator_name, dimension_node_name)
        
        dimensions.append((dimension_node_name, axis_name))
    
    return dimensions

# Create Qt application
app = QApplication.instance()
if not app:
    app = QApplication([])
    
# Create and show the dialog
dialog = AxisSelectionDialog()
dialog.show()